package de.hdm_stuttgart.mi.sd1.task1;

import java.util.Arrays;

/**
 * <p>Array related helper method</p>
 */
public class C_ArrayHelper {

    /**
     * <p>Collect all values from a given array being above its average value. The following example
     * features an array of five values resulting in an average of 6. Only the values 8, 12 and 8
     * are strictly above average:</p>
     *
     * <table class="goikTableDefaults">
     *     <caption>Finding values above average</caption>
     *     <tbody>
     *         <tr>
     *             <th>Code</th>
     *             <th>Result</th>
     *         </tr>
     *         <tr>
     *             <td>
     *                 <pre><code class="java"> final int[] values = {-4, 6, 8, 12, 8};
     * System.out.println( "Above average: " +
     *     Arrays.toString(getAboveAverage(values)));</code></pre>
     *             </td>
     *             <td>
     *                 <pre><code class="nohighlight"> Above average: [8, 12, 8] </code></pre>
     *             </td>
     *         </tr>
     *     </tbody>
     * </table>
     *
     * @param values A list of values.
     * @return All values from the list being strictly above the list's average thereby preserving
     * the original array's order of appearance.
     */
    static public int[] getAboveAverage (int[] values) {

        // Step 1: Calculate the sum of all values
        long sum = 0;  // Avoiding int overflow
        for (final int v: values) {
            sum += v;
        }

        // Step 2: Create a »shadow« array and insert all values above average
        //
        final int allValuesCount = values.length;
        final int[] result = new int[allValuesCount - 1 /* At least one value is below average */];
        int lastAboveValueIndex = 0;

        for (final int v: values) {
            // Multiply by count of values rather than calculating the average thereby avoiding rounding problems.
            //
            if (sum < allValuesCount * ((long) v) /* avoiding int overflow */) {
                result[lastAboveValueIndex++] = v;
            }
        }

        // Step three: Shorten shadow array to actual values above average.
        //
        return Arrays.copyOf(result, lastAboveValueIndex);
    }
    private C_ArrayHelper(){/* Ignore me: My sole purpose is suppressing default constructor javadoc generation */}
}
